# Installer Functions
# Comments to this script have been added upon request by the chocolatey community reviewer
function CreateShortcut ($shortcutObject) {
  if($shortcutObject.execute -eq $true) {
    $tempShortcutPathAndName = "" + $shortcutObject.linkPath + "\" + $shortcutObject.linkName
    If (-not(Test-Path -Path $shortcutObject.destPath)) {
      Write-Debug "Target path does not exist. Creating it first."
      New-Item -ItemType Directory -Force -Path $shortcutObject.destPath
    }
    Write-Debug "This is the Shortcut Full Path:"; Write-Debug $tempShortcutPathAndName
    Install-ChocolateyShortcut -ShortcutFilePath $tempShortcutPathAndName -TargetPath $shortcutObject.destPath
  }
}

function CreateSymlink ($symlinkObject) {
  if($symlinkObject.execute -eq $true) {
    $tempSymlinkPathAndName = $symlinkObject.linkPath + "\" + $symlinkObject.linkName
    If ($tempSymlinkPathAndName -eq $symlinkObject.destPath) {
      Write-Error ("Target and Destination Paths of an overwritten parameter and an internal parameter are the same.")
      return
    }
    if (Test-Path -Path $symlinkObject.destPath) {
      Write-Debug ("Destination Path " + $symlinkObject.destPath + " already exists.")
    } else {
      Write-Debug ("Creating " + $symlinkObject.destPath)
      New-Item $symlinkObject.destPath -ItemType Directory -Force
    }
    if (Test-Path -Path $tempSymlinkPathAndName) {
      if((Get-Item $tempSymlinkPathAndName).Attributes.ToString() -Match "ReparsePoint") {
        Write-Debug ("Link Path " + $tempSymlinkPathAndName + " already exists as SymbolicLink. Will overwrite SymbolicLink.")
      } else {
        Write-Debug ("Link Path " + $tempSymlinkPathAndName + " already exists. Will move to destination path and link afterwards.")
        If (Test-Path $tempSymlinkPathAndName -pathType leaf) {
          Write-Debug ("Link Path " + $tempSymlinkPathAndName + " is a file.")
          Move-Item -Path $tempSymlinkPathAndName -Destination $symlinkObject.destPath -Force
        } ElseIf (Test-Path $tempSymlinkPathAndName -pathType container) {
          Write-Debug ("Link Path " + $tempSymlinkPathAndName + " is a folder.")
          Move-Item -Path "$tempSymlinkPathAndName\*" -Destination $symlinkObject.destPath -Force
          Remove-Item -Path "$tempSymlinkPathAndName"
        } Else {
          Write-Error ("Link Path " + $tempSymlinkPathAndName + " is neither file nor folder.")
          return
        }
      }
    }
    Write-Debug ("Linking " + $tempSymlinkPathAndName + " to " + $symlinkObject.destPath)
    New-Item -Type SymbolicLink -Path "$tempSymlinkPathAndName" -Value $symlinkObject.destPath -Force
  }
}

function CreateRegKey ($regKeyObject) {
  if($regKeyObject.execute -eq $true) {
    if($regKeyObject.key -eq '') {
      New-Item -Path ${regKeyObject}.path -Value ${regKeyObject}.value -Force
    } else {
      if (Test-Path ${regKeyObject}.path) { } else { New-Item ${regKeyObject}.path -force }
      New-ItemProperty -Type String -Path ${regKeyObject}.path -Name ${regKeyObject}.key -Value ${regKeyObject}.value -force
    }
  }
}

function CreateRegKeyFromFile ($regKeyFileObject) {
  if($regKeyFileObject.execute -eq $true) {
    Write-Debug ("Creating registry entry from file " + $env:ChocolateyPackageFolder + "\" + $regKeyFileObject.path)
    regedit.exe /S ($env:ChocolateyPackageFolder + "\" + $regKeyFileObject.path)
  }
}

<#
.SYNOPSIS
Create the component list that will be parsed to the installer.

.DESCRIPTION
Create the component list that will be parsed to the installer. A component is a tickable box in an installer.
The resulting component list depends on the 'execute' bool that should be set before handing the installer
component into this function.

.PARAMETER instParamObject
An installer parameter object/action defined in the chocolateyvariables.ps1.

.NOTES
Comment added because reviewer asked to do so.
#>
function CreateInstallerParameters ($instParamObject) {
  if($instParamObject.execute -eq $true) {
    Write-Debug "Checking Installer Components for: "; Write-Debug $instParamObject.value
    $global:installerComponents += $instParamObject.value
    $global:installerComponents += ","
    Write-Debug ("These are the current installerComponents: " + $installerComponents + ".")
  }
}

function CopyPackageRessources ($PackageResourcePathObject) {
  if($PackageResourcePathObject.execute -eq $true) {
    Write-Debug ("Preparing Package Ressource Path for: " + $PackageResourcePathObject.value + ".")
    $tempPath = ($env:ChocolateyPackageFolder + "\" + ($PackageResourcePathObject.value).ToString())
    if((Test-Path $tempPath) -eq $false) {
      Write-Debug ("The resource object " + $PackageResourcePathObject.value + " does not exist.")
      return
    }
    # We create the target directory first as Copy-Item behaves differently when the target path doesn't exist as directory
    New-Item $PackageResourcePathObject.installPath -ItemType Directory -Force
    if($tempPath -Match "zip") {
      Get-ChocolateyUnzip -FileFullPath "$tempPath" -Destination $PackageResourcePathObject.installPath
    } else {
      Move-Item -Path $tempPath -Destination $PackageResourcePathObject.installPath -Force -Recurse
      Write-Debug ("Copying all files from Package\" + $PackageResourcePathObject.value + " to " + $PackageResourcePathObject.installPath + ".")
    }
  }
}

function CreateTxtFiles ($txtFileObject) {
  if($txtFileObject.execute -eq $true) {
    Write-Debug ("Creating txt file in " + $txtFileObject.key + " with the content: " + $txtFileObject.value)
    Remove-Item $txtFileObject.key -ErrorAction SilentlyContinue
    New-Item $txtFileObject.key -ItemType file -Force
    [System.IO.File]::WriteAllLines($txtFileObject.key, $txtFileObject.value, (New-Object ("System.Text." + $txtFileObject.encoding + "Encoding") $txtFileObject.bom))
  }
}

function RunInstallerWithPackageParametersObject ($packageParameterObject) {
  $installerPathViaPP = (Test-Path $pp["InstallerPath"])
  ## Does the following check work for 32 bit installers??
  $installerEmbedded = ((Test-Path ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file)) -And ($packageParameterObject.file64 -ne $null) -And (Test-Path ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64)))
  $installerDownload = (($packageParameterObject.url -ne $null) -Or ($packageParameterObject.url64 -ne $null))
  $installerDownloadExe = (($packageParameterObject.url -ne $null) -And ($packageParameterObject.url).EndsWith(".exe"))
  Write-Debug ("This is the InstallerPath Variable: " + $pp["InstallerPath"])
  Write-Debug ("Installer embedded path: " + ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file))
  Write-Debug ("Installer embedded path: " + ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64))
  if(Test-Path variable:packageParameterObject.url]) {Write-Debug ("Installer url: " + $packageParameterObject.url)}
  if($installerDownloadExe -eq $true) {Write-Debug "Installer is exe"}

  if($installerPathViaPP -eq $true) {
    $packageParameterObject["file"] = $fileLocation
    Install-ChocolateyInstallPackage @packageParameterObject # https://chocolatey.org/docs/helpers-install-chocolatey-install-package
  }

  if($installerEmbedded -eq $true) {
    $packageParameterObject.file = ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file)
    $packageParameterObject.file64 = ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64)
    Write-Debug ("Installer embedded path: " + $packageParameterObject.file)
    Install-ChocolateyInstallPackage @packageParameterObject
  }

  if($installerDownload -eq $true -And $installerPathViaPP -eq $false) {
    if($installerDownloadExe -eq $true) { "Installer is exe, running now..."
      Install-ChocolateyPackage @packageParameterObject # https://chocolatey.org/docs/helpers-install-chocolatey-package
    } else { Write-Debug "Installer inside zip"; Write-Debug ("UnzipLocation is: " + $packageParameterObject.unzipLocation)
      $packageParameterObject["file"] = $fileLocation
      Install-ChocolateyZipPackage @packageParameterObject
      Install-ChocolateyInstallPackage @packageParameterObject
    }
  }
}

function RemoveInstallerObjects ($packageParameterObject) {
  if($packageParameterObject.file) {
    if(Test-Path ($packageParameterObject.file) -ErrorAction Ignore){
      Remove-Item $packageParameterObject.file -Force -ErrorAction SilentlyContinue
    }
  }
  if($packageParameterObject.file64) {
    if(Test-Path ($packageParameterObject.file64) -ErrorAction Ignore){
      Remove-Item $packageParameterObject.file64 -Force -ErrorAction SilentlyContinue
    }
  }
}

function RemoveTemporaryFiles ($PackageResourcePathObject) {
  Write-Debug ("Deleting temp package files: " + $env:ChocolateyPackageFolder + "\" + $PackageResourcePathObject.value + " and" + $env:ChocolateyPackageFolder + "\" + $PackageResourcePathObject.value + ".txt")
  Remove-Item ($env:ChocolateyPackageFolder + "\" + $PackageResourcePathObject.value) -Force -ErrorAction SilentlyContinue
  Remove-Item ($env:ChocolateyPackageFolder + "\" + $PackageResourcePathObject.value + ".txt") -Force -ErrorAction SilentlyContinue
}

function HandlePackageArgs ($packageParameterObject) {
  if ($installerComponents) {
    $packageParameterObject["silentArgs"] += " /Components=$installerComponents" #Q2 2018 the '' around $installerComponents have to be removed because Zebra2.8 installer ignores parameters
  }
  Write-Debug "This are the silentArgs"; Write-Debug $packageParameterObject["silentArgs"]
}

# Creates txt file containing all zip files and where they have been copied to.
# Also checks whether extracted zip content should be uninstalled or not (.delete=$false).
function WriteUninstallData ($PackageResourcePathObject) {
  if($PackageResourcePathObject.delete -ne $false) {
    foreach($line in Get-Content ($env:ChocolateyPackageFolder + "\" + $PackageResourcePathObject.value + ".txt")) {
      Add-Content -Path ($env:ChocolateyPackageFolder + "\uninstall.txt") -Value ($PackageResourcePathObject.installPath + "\" + $line)
    }
  }
}

# Creates/Overwrites bit-aware variables, so x86 files can use the same path-var on both platforms
function CreateBitAwareVariables () {
  if($osBitness -eq 64) {
    $global:vst2x86BitAware = "$vst2x86_64Path"
    $global:vst3x86BitAware = "$vst3x86_64Path"
    $global:aaxx86BitAware  = "$aaxx86_64Path"
    $global:vst2x86PathRegBitAware = "$vst2x86_64PathReg"
  }
  if($osBitness -eq 32) {
    $global:vst2x86BitAware = "$vst2Path"
    $global:vst3x86BitAware = "$vst3Path"
    $global:aaxx86BitAware  = "$aaxPath"
    $global:vst2x86PathRegBitAware = "$vst2PathReg"
  }
}

# Uninstaller Functions
function Test-Registry ($path, $value) {
  try {
    Get-ItemProperty -Path $Path | Select-Object -ExpandProperty $Value -ErrorAction Stop | Out-Null
    return $true
  } catch {
    return $false
  }
}

function DeleteFolders ($folderPath) {
  if([string]::IsNullOrWhiteSpace($folderPath)) {
    Write-Debug "Path variable is null."
    return
  }
  if(Test-Path -path "$folderPath") {
    Write-Debug "$folderPath will be deleted."
    # "Archive" seems to be a lnk file
    if((Get-Item ($folderPath).ToString()).Attributes.ToString() -Match "Archive" ) {
      Remove-Item $folderPath -Force -Recurse
    } else {
      [System.IO.Directory]::Delete("$folderPath", $true)
    }
  }
}

# Checks, if a folder is empty and if true, deletes that folder. Also checks three levels upwards. -Force is not necessary.
function DeleteFoldersIfEmpty ($folderPath) {
  Write-Debug ("Checking " + $folderPath)
  if([string]::IsNullOrWhiteSpace($folderPath)) {
    Write-Debug "Path variable is null."
    return
  }
  if((Test-Path -Path "$folderPath") -eq $false) {
    Write-Debug "$folderPath does not exist."
    return
  }
  $parentFolder = (Get-ItemProperty $folderPath).Parent.FullName
  $parentParentFolder = (Get-ItemProperty $folderPath).Parent.Parent.FullName
  $folderList = "$folderPath", "$parentFolder", "$parentParentFolder"
  Foreach ($path in $folderList) {
    Write-Debug ("Checking if path exists and is empty: " + $path)
    if($path -ne '' -and $path -ne $null) {
      if(((Test-Path "$path") -and (Get-ChildItem "$path") -eq $null)) {
        Write-Debug "$path is empty and will be deleted."
        Remove-Item "$path"
      }
    }
  }
}

function DeleteDataFromTxtFile ($uninstallFile) {
  Foreach($path in Get-Content $uninstallFile) {
    If ([string]::IsNullOrWhiteSpace($path)) {
      return
    }
    Write-Debug ("About to delete " + $path)
    if(Test-Path $path -pathType leaf) {
      $directoryName = (Get-Item $path).DirectoryName
      Write-Debug "$path will be deleted."
      Remove-Item "$path" -Force
      DeleteFoldersIfEmpty($directoryName)
    }
    if(Test-Path $path -pathType container) {
      $parentDirectory = (Get-ItemProperty $path).Parent.FullName
      Write-Debug "$path will be deleted."
      #Remove-Item -Recurse -Force fails if folder contains a symlink -.-
      [System.IO.Directory]::Delete("$path", $true)
      DeleteFoldersIfEmpty($parentDirectory)
    }
  }
}

# Removes all registry entires in a given registry file from the registry
function DeleteRegKeysFromFile ($file) {
  if(Test-Path $file) {
    (Get-Content $file) -replace "^\[","[-" | out-file $file
    regedit.exe /S ($file)
  }
}

# Deletes a registry key or property, if it exists and is not marked as delete=$false
function DeleteRegKeyFromObjects ($regKey) {
  Write-Debug ("About to delete Registry Entry: " + $regKey.path + $regKey.key)
  if($regKey.ContainsKey("delete")) {
    return
  }
  If ([string]::IsNullOrWhiteSpace($regKey.path)) {
    Write-Debug("Given Registry Path is null or empty.")
    return
  }
  if((Test-Path -Path $regKey.path) -eq $false) {
    Write-Debug ("Registry Entry does not exist anymore.")
    return
  }
  if(($regKey.key -eq "")) {
    Write-Debug ("Deleting " + $regKey.path)
    Remove-Item -Path $regKey.path -Force
  }
  if ($regKey.key -ne "") {
    Write-Debug ("Deleting Key " + $regKey.key + " at Path " + $regKey.path)
    Remove-ItemProperty -Path $regKey.path -Name $regKey.key -Force -ErrorAction SilentlyContinue
  }
}

# Deletes a file, if it exists
function DeleteFile ($pathToFile) {
  Write-Debug ("Checking " + $pathToFile)
  if(Test-Path $pathToFile) {
    Write-Debug ("Deleting " + $pathToFile)
    Remove-Item $pathToFile -Force
  }
}

# Packaging functions
function CreateFileList ($packagePaths, $targetPath) {
  Remove-Item ($targetPath + ".txt") -ErrorAction SilentlyContinue
  Foreach ($path in $packagePaths) {
    #Add-Content -Path ($targetPath + ".txt") -Value (Get-ChildItem -Path "$path" -Recurse -Name) -Force
    Add-Content -Path ($targetPath + ".txt") -Value ((Get-Item -Path "$path").Name) -Force
  }
}
function CreateDataArchive ($packagePaths, $targetPath) {
  Compress-Archive $packagePaths $targetPath -CompressionLevel Optimal -Force
}

function CreateDataArchiveAndFileList ($packagePaths, $targetPath) {
  if ((Test-Path $targetPath) -eq $false ) {
    New-Item $targetPath -Force
    Remove-Item $targetPath -Force
  }
  CreateDataArchive $packagePaths $targetPath
  CreateFileList $packagePaths $targetPath
}

<#
.SYNOPSIS
Reduces the nupkg package file by deleting big binaries from the file.

.DESCRIPTION
  Reduces the nupkg package file by deleting all files from the .nupkg file that are not necessary anymore like installer files or 7z binaries.
  It keeps the following files and folders:
  /package/
  /tools/
  _rels
  *.nuspec
  [Content_Types].xml

  The rest will be removed from the nupkg file.

.NOTES
Comment added because reviewer asked to do so.
#>
#
function ReducePackageSize () {
  .$env:ChocolateyInstall\tools\7z.exe d ($env:ChocolateyPackageFolder + "\" + $env:ChocolateyPackageName + ".nupkg")  * -r  -xr!package -xr!tools -xr!_rels -x!"*.nuspec" -x!"[Content_Types].xml"
}
<#
.SYNOPSIS
Which installation steps are actually necessary for the given OS and package parameters?

.DESCRIPTION
Checks the current OS bitness and the given package parameters and marks all steps from the chocolateyvariables file to be actually executed by setting their 'execute' bit.

.PARAMETER objectsList
A list of objects with a 'validpp' key and a 'bit' key from the chocolateyvariables.ps1 file.

.NOTES
Comment added because reviewer asked to do so.
#>

function DetermineExecutionOfAllObjects ($objectsList) {
  Foreach($object in $objectsList) {
    $object.Add('execute', $false)
    $bitStatus = ($object.bit -eq $null) -Or ($object.bit -Contains $osBitness)
    $ppIndependent = (($object.validpp -eq $null) -Or ($object.validpp -Like "Always"))
    Write-Debug ("ppindependent: " + $ppIndependent)
    Write-Debug ("bitStatus: " + $bitStatus)

    Foreach($packageParameter in $object.validpp) {
      Write-Debug ("Status of evaluated package parameter:" + $packageParameter)
      if((Test-Path variable:pp.packageParameter) -eq $true) {
        Write-Debug $pp[$packageParameter]
        Write-Debug ($pp[$packageParameter] -eq $false)
      }
      if(($pp[$packageParameter] -eq $false) -And $bitStatus) {
        $object.execute = $true
      }
    }

    if($ppIndependent -And $bitStatus) {
      $object.execute = $true
    }
  }
}

<#
.SYNOPSIS
Create the protocol file for the uninstallation

.DESCRIPTION
Writes a list of files that have been extracted from zip files embedded in this package. Is empty if no zip files have been embedded.

.NOTES
Comment added because reviewer asked to do so.
#>
function CreateUninstallFile () {
  # Create empty uninstall file
Out-File -FilePath ($env:ChocolateyPackageFolder + "\uninstall.txt")
}
